"use strict";

/* exported logger */
var Promise = require("bluebird");

var AbstractGuiElement = require('../AbstractGuiElement');
var AddOverlayDialog = require('../AddOverlayDialog');
var Annotation = require('../../map/data/Annotation');
var CommentsTab = require('./CommentsAdminPanel');
var GuiConnector = require('../../GuiConnector');
var PrivilegeType = require('../../map/data/PrivilegeType');
var ValidationError = require("../../ValidationError");

var Functions = require('../../Functions');
// noinspection JSUnusedLocalSymbols
var logger = require('../../logger');

var guiUtils = new (require('../leftPanel/GuiUtils'))();
var xss = require('xss');

/**
 *
 * @param params
 * @param {HTMLElement} params.element
 * @param {Configuration} params.configuration
 * @param {ServerConnector} params.serverConnector
 *
 * @constructor
 * @extends {AbstractGuiElement}
 */
function EditProjectDialog(params) {
  AbstractGuiElement.call(this, params);
  var self = this;
  self.setConfiguration(params.configuration);
  self.registerListenerType("onSave");
  $(self.getElement()).addClass("minerva-edit-project-dialog");
  $(self.getElement()).css({overflow: "hidden"});

  self.createGui();
}

EditProjectDialog.prototype = Object.create(AbstractGuiElement.prototype);
EditProjectDialog.prototype.constructor = EditProjectDialog;

/**
 *
 */
EditProjectDialog.prototype.createGui = function () {
  var self = this;

  guiUtils.initTabContent(self);

  guiUtils.addTab(self, {
    name: "GENERAL",
    content: self.createGeneralTabContent()
  });

  guiUtils.addTab(self, {
    name: "OVERLAYS",
    content: self.createOverlaysTabContent()
  });
  guiUtils.addTab(self, {
    name: "MAPS",
    content: self.createMapsTabContent()
  });
  guiUtils.addTab(self, {
    name: "USERS",
    content: self.createUsersTabContent()
  });

  guiUtils.addTab(self, {
    name: "COMMENTS",
    panelClass: CommentsTab
  });
  self.setCommentsTab(self._panels[0]);

  $(".nav-tabs > li > a", self.getElement()).bind("click", function () {
    //workaround for some css issues...
    $(".parentTabs", self.getElement()).css("top", "40px");
    $(".parentTabs", self.getElement()).css("bottom", "10px");
  });
};

/**
 *
 * @returns {HTMLElement}
 */
EditProjectDialog.prototype.createGeneralTabContent = function () {
  var self = this;

  var result = Functions.createElement({
    type: "div",
    className: "minerva-project-general-tab"
  });

  var table = Functions.createElement({
    type: "div",
    style: "display:table"
  });
  result.appendChild(table);

  var projectIdRow = Functions.createElement({
    type: "div",
    style: "display:table-row"
  });
  table.appendChild(projectIdRow);
  projectIdRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "ProjectId"
  }));
  projectIdRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    name: "projectId"
  }));

  var mapCanvasTypeRow = Functions.createElement({
    type: "div",
    style: "display:table-row"
  });
  table.appendChild(mapCanvasTypeRow);
  mapCanvasTypeRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "Map canvas type: "
  }));
  mapCanvasTypeRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: Functions.createElement({
      type: "select",
      name: "project-map-canvas-type"
    })
  }));

  var googleLicenseRow = Functions.createElement({
    type: "div",
    style: "display:table-row"
  });
  table.appendChild(googleLicenseRow);
  googleLicenseRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "I agree to <a target='_license' href='https://developers.google.com/maps/terms'>Google Maps APIs Terms of Service</a>: ",
    xss: false
  }));
  googleLicenseRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: Functions.createElement({
      type: "input",
      inputType: "checkbox",
      name: "project-google-maps-license"
    })
  }));

  var nameRow = Functions.createElement({
    type: "div",
    style: "display:table-row"
  });
  table.appendChild(nameRow);
  nameRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "Name"
  }));
  nameRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "<input name='projectName'/>",
    xss: false
  }));

  var versionRow = Functions.createElement({
    type: "div",
    style: "display:table-row"
  });
  table.appendChild(versionRow);
  versionRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "Version"
  }));
  versionRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "<input name='projectVersion'/>",
    xss: false
  }));

  var diseaseRow = Functions.createElement({
    type: "div",
    style: "display:table-row"
  });
  table.appendChild(diseaseRow);
  diseaseRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "Disease"
  }));
  diseaseRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "<input name='projectDisease'/>",
    xss: false
  }));

  var organismRow = Functions.createElement({
    type: "div",
    style: "display:table-row"
  });
  table.appendChild(organismRow);
  organismRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "Organism"
  }));
  organismRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "<input name='projectOrganism'/>",
    xss: false
  }));

  var emailRow = Functions.createElement({
    type: "div",
    style: "display:table-row"
  });
  table.appendChild(emailRow);
  emailRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "Notify email"
  }));
  emailRow.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "<input name='projectNotifyEmail'/>",
    xss: false
  }));

  var menuRow = Functions.createElement({
    type: "div",
    className: "minerva-menu-row",
    style: "display:table-row; margin:10px"
  });
  result.appendChild(menuRow);

  var saveProjectButton = Functions.createElement({
    type: "button",
    name: "saveProject",
    content: '<span class="ui-icon ui-icon-disk"></span>&nbsp;SAVE',
    onclick: function () {
      return self.onSaveClicked().then(function () {
        return self.callListeners("onSave");
      }).then(function () {
        return self.close();
      }).catch(GuiConnector.alert);
    },
    xss: false
  });
  var cancelButton = Functions.createElement({
    type: "button",
    name: "cancelProject",
    content: '<span class="ui-icon ui-icon-cancel"></span>&nbsp;CANCEL',
    onclick: function () {
      return self.close();
    },
    xss: false
  });
  menuRow.appendChild(saveProjectButton);
  menuRow.appendChild(cancelButton);

  return result;

};

/**
 *
 * @returns {HTMLElement}
 */
EditProjectDialog.prototype.createMapsTabContent = function () {
  var self = this;
  var result = Functions.createElement({
    type: "div",
    className: "minerva-maps-tab"
  });
  result.appendChild(self._createMapsTable());
  return result;
};

EditProjectDialog.prototype._createMapsTable = function () {
  var self = this;

  var result = Functions.createElement({
    type: "div",
    style: "margin-top:10px;"
  });

  var mapsTable = Functions.createElement({
    type: "table",
    name: "mapsTable",
    className: "display",
    style: "width:100%"
  });
  result.appendChild(mapsTable);

  $(mapsTable).DataTable({
    fnRowCallback: function (nRow, aData) {
      nRow.setAttribute('id', "map-" + aData[0]);
    },
    columns: [{
      title: 'Id'
    }, {
      title: 'Name'
    }, {
      title: 'Default center x'
    }, {
      title: 'Default center y'
    }, {
      title: 'Default zoom level'
    }, {
      title: 'Update'
    }]
  });

  $(mapsTable).on("click", "[name='saveMap']", function () {
    var button = this;
    GuiConnector.showProcessing("Updating");
    return self.saveMap(parseInt($(button).attr("data"))).then(function () {
      GuiConnector.hideProcessing();
      GuiConnector.info("Map updated successfully");
    }, function (error) {
      GuiConnector.hideProcessing();
      GuiConnector.alert(error);
    });
  });

  return result;
};

/**
 *
 * @returns {HTMLElement}
 */
EditProjectDialog.prototype.createOverlaysTabContent = function () {
  var self = this;
  var result = Functions.createElement({
    type: "div",
    className: "minerva-project-overlays-tab"
  });
  result.appendChild(self._createOverlayTable());
  return result;
};

/**
 *
 * @returns {HTMLElement}
 * @private
 */
EditProjectDialog.prototype._createOverlayTable = function () {
  var self = this;

  var result = Functions.createElement({
    type: "div",
    style: "margin-top:10px;"
  });

  var overlaysTable = Functions.createElement({
    type: "table",
    name: "overlaysTable",
    className: "display",
    style: "width:100%"
  });
  result.appendChild(overlaysTable);

  // noinspection JSUnusedGlobalSymbols
  $(overlaysTable).DataTable({
    fnRowCallback: function (nRow, aData) {
      nRow.setAttribute('id', "overlay-" + aData[0]);
    },
    columns: [{
      title: 'Id'
    }, {
      title: 'Name'
    }, {
      title: 'Description'
    }, {
      title: 'Public'
    }, {
      title: 'Default'
    }, {
      title: 'Owner'
    }, {
      title: 'Data'
    }, {
      title: 'Update'
    }, {
      title: 'Remove'
    }],
    dom: '<"minerva-datatable-toolbar">frtip',
    initComplete: function () {
      $("div.minerva-datatable-toolbar", $(result)).html('<button name="addOverlay">Add overlay</button>');
    }

  });

  $(overlaysTable).on("click", "[name='removeOverlay']", function () {
    var button = this;
    return self.removeOverlay(parseInt($(button).attr("data"))).then(null, GuiConnector.alert);
  });

  $(overlaysTable).on("click", "[name='saveOverlay']", function () {
    var button = this;
    GuiConnector.showProcessing("Updating");
    return self.saveOverlay(parseInt($(button).attr("data"))).then(function () {
      GuiConnector.hideProcessing();
      GuiConnector.info("Overlay updated successfully");
    }, function (error) {
      GuiConnector.hideProcessing();
      GuiConnector.alert(error);
    });
  });

  $(overlaysTable).on("click", "[name='downloadSource']", function () {
    var button = this;
    return self.getServerConnector().getOverlaySourceDownloadUrl({
      overlayId: $(button).attr("data")
    }).then(function (url) {
      return self.downloadFile(url);
    }).then(null, GuiConnector.alert);
  });

  $(result).on("click", "[name='addOverlay']", function () {
    return self.openAddOverlayDialog();
  });

  return result;
};

/**
 *
 * @returns {HTMLElement}
 */
EditProjectDialog.prototype.createUsersTabContent = function () {
  var self = this;

  var result = Functions.createElement({
    type: "div",
    className: "minerva-project-users-tab",
    style: "margin-top:10px;"
  });

  var usersTable = Functions.createElement({
    type: "table",
    name: "usersTable",
    className: "display",
    style: "width:100%"
  });
  result.appendChild(usersTable);

  $(usersTable).on("click", "[name='saveUser']", function () {
    var button = this;
    GuiConnector.showProcessing("Updating");
    return self.saveUser($(button).attr("data")).then(function () {
      GuiConnector.hideProcessing();
      GuiConnector.info("User updated successfully");
    }, function (error) {
      GuiConnector.hideProcessing();
      GuiConnector.alert(error);
    });
  });

  return result;
};

/**
 *
 * @returns {{title: string, privilegeType: PrivilegeType|undefined}[]}
 */
EditProjectDialog.prototype.createUserPrivilegeColumns = function () {
  var self = this;

  if (self._userPrivilegeColumns !== undefined) {
    return self._userPrivilegeColumns;
  }

  var configuration = self.getConfiguration();
  self._userPrivilegeColumns = [{
    title: "Name"
  }];
  var privilegeTypes = configuration.getPrivilegeTypes();
  for (var i = 0; i < privilegeTypes.length; i++) {
    var type = privilegeTypes[i];
    if (type.getObjectType() === "Project") {
      self._userPrivilegeColumns.push({
        "title": type.getCommonName(),
        privilegeType: type
      });
    }
  }
  self._userPrivilegeColumns.push({
    "title": "Update"
  });
  return self._userPrivilegeColumns;
};

/**
 *
 * @returns {Promise}
 */
EditProjectDialog.prototype.init = function () {
  var self = this;
  var select = $("[name='project-map-canvas-type']", self.getElement())[0];
  var licenceCheckbox = $("[name='project-google-maps-license']", self.getElement())[0];
  var licenceDiv = licenceCheckbox.parentElement.parentElement;

  $(select).change(function () {
    $("option:selected", select).each(function () {
      if ($(this).val() === "GOOGLE_MAPS_API") {
        $(licenceDiv).css("display", "table-row");
      } else {
        $(licenceDiv).css("display", "none");
      }
      self.setLicenseAccepted(false);
    });
  });

  return self.initUsersTab().then(function () {
    return self.refresh()
  }).then(function () {
    $(window).trigger('resize');
  });
};

/**
 *
 * @returns {Promise}
 */
EditProjectDialog.prototype.refresh = function () {
  var self = this;
  var element = self.getElement();
  var project = self.getProject();
  $("[name='projectName']", element).val(xss(project.getName()));
  $("[name='projectId']", element).val(project.getProjectId());
  $("[name='projectVersion']", element).val(xss(project.getVersion()));

  var disease = "";
  if (project.getDisease() !== undefined) {
    disease = xss(project.getDisease().getResource());
  }
  $("[name='projectDisease']", element).val(disease);
  var organism = "";
  if (project.getOrganism() !== undefined) {
    organism = xss(project.getOrganism().getResource());
  }
  $("[name='projectOrganism']", element).val(organism);

  var email = "";
  if (project.getNotifyEmail() !== undefined) {
    email = xss(project.getNotifyEmail());
  }
  $("[name='projectNotifyEmail']", element).val(email);


  return self.refreshUsers().then(function () {
    return self.refreshMaps();
  }).then(function () {
    return self.refreshOverlays();
  }).then(function () {
    return self.refreshComments();
  }).then(function () {
    var configuration = self.getConfiguration();
    var mapCanvasTypes = configuration.getMapCanvasTypes();
    var select = $("[name='project-map-canvas-type']", self.getElement())[0];
    $(select).empty();
    var licenceCheckbox = $("[name='project-google-maps-license']", self.getElement())[0];
    var licenceDiv = licenceCheckbox.parentElement.parentElement;

    for (var i = 0; i < mapCanvasTypes.length; i++) {
      var mapCanvasType = mapCanvasTypes[i];
      var option = Functions.createElement({
        type: "option",
        content: mapCanvasType.name,
        value: mapCanvasType.id
      });
      if (mapCanvasType.id === self.getProject().getMapCanvasType()) {
        option.selected = true;
        if (mapCanvasType.id === "GOOGLE_MAPS_API") {
          $(licenceDiv).css("display", "table-row");
          self.setLicenseAccepted(true);
        } else {
          $(licenceDiv).css("display", "none");
          self.setLicenseAccepted(false);
        }
      }
      select.appendChild(option);
    }
  });
};

/**
 *
 * @returns {Promise}
 */
EditProjectDialog.prototype.initUsersTab = function () {
  var self = this;

  var usersTable = $("[name=usersTable]", self.getElement())[0];

  var columns = self.createUserPrivilegeColumns();
  $(usersTable).DataTable({
    columns: columns
  });
  return Promise.resolve();
};

/**
 *
 * @returns {Promise}
 */
EditProjectDialog.prototype.refreshOverlays = function () {
  var self = this;
  return self.getServerConnector().getLoggedUser().then(function (user) {
    var privilege = self.getConfiguration().getPrivilegeType(PrivilegeType.USER_MANAGEMENT);
    //we need to refresh users as well because of privileges
    if (user.hasPrivilege(privilege)) {
      return self.getServerConnector().getOverlays({
        projectId: self.getProject().getProjectId()
      }).then(function (overlays) {
        return self.setOverlays(overlays);
      });
    } else {
      guiUtils.hideTab(self, $(".minerva-project-overlays-tab", self.getElement())[0]);
    }
  });
};

/**
 *
 * @returns {Promise}
 */
EditProjectDialog.prototype.refreshMaps = function () {
  var self = this;
  return self.getServerConnector().getModels(self.getProject().getProjectId()).then(function (maps) {
    return self.setMaps(maps);
  });
};

/**
 *
 * @returns {Promise}
 */
EditProjectDialog.prototype.refreshUsers = function () {
  var self = this;
  return self.getServerConnector().getLoggedUser().then(function (user) {
    var privilege = self.getConfiguration().getPrivilegeType(PrivilegeType.USER_MANAGEMENT);
    //we need to refresh users as well because of privileges
    if (user.hasPrivilege(privilege)) {
      return ServerConnector.getUsers(true).then(function (users) {
        return self.setUsers(users);
      });
    } else {
      guiUtils.hideTab(self, $(".minerva-project-users-tab", self.getElement())[0]);
    }
  });
};

/**
 *
 * @param {DataOverlay[]} overlays
 * @returns {Promise}
 */
EditProjectDialog.prototype.setOverlays = function (overlays) {
  var self = this;
  self._overlayById = [];
  return ServerConnector.getUsers().then(function (users) {
    var dataTable = $($("[name='overlaysTable']", self.getElement())[0]).DataTable();
    var data = [];
    for (var i = 0; i < overlays.length; i++) {
      var overlay = overlays[i];
      self._overlayById[overlay.getId()] = overlay;
      var rowData = self.overlayToTableRow(overlay, users);
      data.push(rowData);
    }
    dataTable.clear().rows.add(data).draw();
  });
};

/**
 *
 * @param {MapModel[]} maps
 */
EditProjectDialog.prototype.setMaps = function (maps) {
  var self = this;
  self._mapsById = [];
  var dataTable = $($("[name='mapsTable']", self.getElement())[0]).DataTable();
  var data = [];
  for (var i = 0; i < maps.length; i++) {
    var map = maps[i];
    self._mapsById[map.getId()] = map;
    var rowData = self.mapToTableRow(map);
    data.push(rowData);
  }
  dataTable.clear().rows.add(data).draw();
};

/**
 *
 * @param {User[]} users
 */
EditProjectDialog.prototype.setUsers = function (users) {
  var self = this;
  self._userByLogin = [];
  var columns = self.createUserPrivilegeColumns();
  var dataTable = $($("[name='usersTable']", self.getElement())[0]).DataTable();
  var data = [];
  for (var i = 0; i < users.length; i++) {
    var user = users[i];
    self._userByLogin[user.getLogin()] = user;
    var rowData = self.userToTableRow(user, columns);
    data.push(rowData);
  }
  dataTable.clear().rows.add(data).draw();
};

/**
 *
 * @param {User} user
 * @param {Array} columns
 * @returns {Array}
 */
EditProjectDialog.prototype.userToTableRow = function (user, columns) {
  var self = this;
  var row = [];
  var login = user.getLogin();

  row[0] = user.getName() + " " + user.getSurname() + " (" + login + ")";
  for (var i = 1; i < columns.length; i++) {
    var column = columns[i];
    if (column.privilegeType !== undefined) {
      if (column.privilegeType.getValueType() === "boolean") {
        var checked = '';
        if (user.hasPrivilege(column.privilegeType, self.getProject().getId())) {
          checked = 'checked';
        }
        row[i] = "<input type='checkbox' name='privilege-" + login + "' data='" + column.privilegeType.getName() + "' "
          + checked + "/>";
      } else {
        throw new Error("Unsupported type: " + column.privilegeType.getValueType());
      }
    }
  }

  row.push("<button name='saveUser' data='" + login + "'><i class=\"fa fa-save\" style=\"font-size:17px\"></i></button>");

  return row;
};

/**
 *
 * @param {DataOverlay} overlay
 * @param {User[]} users
 * @returns {Array}
 */
EditProjectDialog.prototype.overlayToTableRow = function (overlay, users) {
  var row = [];
  var id = overlay.getId();
  var creatorSelect;
  if (overlay.getCreator() === "") {
    creatorSelect = "<select name='creator-" + id + "' value=''>";
  } else {
    creatorSelect = "<select name='creator-" + id + "'>";
  }

  creatorSelect += "<option value='' >---</option>";
  for (var i = 0; i < users.length; i++) {
    var selected = "";
    var user = users[i];
    if (overlay.getCreator() === user.getLogin()) {
      selected = "selected";
    } else {
      selected = "";
    }

    creatorSelect += "<option value='" + user.getLogin() + "' " + selected + ">" + user.getLogin() + "("
      + user.getName() + " " + user.getSurname() + ")</option>";
  }
  creatorSelect += "</select>";

  var checked = '';
  if (overlay.getPublicOverlay()) {
    checked = "checked";
  }
  var publicOverlayCheckbox = "<input type='checkbox' name='publicOverlay-" + id + "' " + checked + "/>";

  checked = '';
  if (overlay.isDefaultOverlay()) {
    checked = "checked";
  }
  var defaultOverlayCheckbox = "<input type='checkbox' name='defaultOverlay-" + id + "' " + checked + "/>";

  var downloadSourceButton;
  if (overlay.getInputDataAvailable()) {
    downloadSourceButton = "<button name='downloadSource' data='" + id + "'>"
      + "<span class='ui-icon ui-icon-arrowthickstop-1-s'></span>" + "</button>";
  } else {
    downloadSourceButton = "N/A";
  }

  row[0] = id;
  row[1] = "<input name='name-" + id + "' value='" + overlay.getName() + "'/>";
  row[2] = "<input name='description-" + id + "' value='" + overlay.getDescription() + "'/>";
  row[3] = publicOverlayCheckbox;
  row[4] = defaultOverlayCheckbox;
  row[5] = creatorSelect;
  row[6] = downloadSourceButton;
  row[7] = "<button name='saveOverlay' data='" + id + "'><i class=\"fa fa-save\" style=\"font-size:17px\"></i></button>";
  row[8] = "<button name='removeOverlay' data='" + id + "'><i class='fa fa-trash-o' style='font-size:17px'></button>";

  return row;
};

/**
 *
 * @param value
 * @returns {*}
 */
function getValueOrEmpty(value) {
  if (value !== undefined && value !== null) {
    return value;
  } else {
    return "";
  }
}

/**
 *
 * @param {MapModel} map
 * @returns {Array}
 */
EditProjectDialog.prototype.mapToTableRow = function (map) {
  var row = [];
  var id = map.getId();
  var centerX = getValueOrEmpty(map.getDefaultCenterX());
  var centerY = getValueOrEmpty(map.getDefaultCenterY());
  var zoomLevel = getValueOrEmpty(map.getDefaultZoomLevel());
  row[0] = id;
  row[1] = map.getName();
  row[2] = "<input name='defaultCenterX-" + id + "' value='" + centerX + "'/>";
  row[3] = "<input name='defaultCenterY-" + id + "' value='" + centerY + "'/>";
  var zoomLevelSelect = "<select name='defaultZoomLevel-" + id + "' value='" + zoomLevel + "'>" +
    "<option value=''>---</option>";
  for (var i = map.getMinZoom(); i <= map.getMaxZoom(); i++) {
    var selected = "";
    if (i === zoomLevel) {
      selected = " selected ";
    }
    zoomLevelSelect += "<option value='" + i + "' " + selected + ">" + (i - map.getMinZoom()) + "</option>";
  }
  zoomLevelSelect += "</select>";
  row[4] = zoomLevelSelect;
  row[5] = "<button name='saveMap' data='" + id + "'><i class=\"fa fa-save\" style=\"font-size:17px\"></i></button>";

  return row;
};

/**
 *
 */
EditProjectDialog.prototype.open = function () {
  var self = this;
  var div = self.getElement();
  if (!$(div).hasClass("ui-dialog-content")) {
    $(div).dialog({
      title: self.getProject().getProjectId(),
      width: window.innerWidth / 2,
      height: window.innerHeight / 2
    });
  }
  $(div).dialog("open");
};

/**
 *
 * @param {string} type
 * @param {string} resource
 * @returns {Annotation|null}
 */
var prepareMiriamData = function (type, resource) {
  if (resource === "" || resource === undefined || resource === null) {
    return null;
  } else {
    return new Annotation({
      type: type,
      resource: resource
    });
  }
};

/**
 *
 * @returns {Promise}
 */
EditProjectDialog.prototype.onSaveClicked = function () {
  var self = this;
  var project = self.getProject();
  var element = self.getElement();
  return self.checkValidity().then(function () {
    project.setName($("[name='projectName']", element)[0].value);
    project.setVersion($("[name='projectVersion']", element)[0].value);
    project.setNotifyEmail($("[name='projectNotifyEmail']", element)[0].value);
    var organism = prepareMiriamData("TAXONOMY", $("[name='projectOrganism']", element)[0].value);
    project.setOrganism(organism);
    var disease = prepareMiriamData("MESH_2012", $("[name='projectDisease']", element)[0].value);
    project.setDisease(disease);
    project.setMapCanvasType(self.getMapCanvasTypeId());
    return self.getServerConnector().updateProject(project);
  });
};

/**
 *
 * @returns {Promise}
 */
EditProjectDialog.prototype.checkValidity = function () {
  var self = this;
  var isValid = true;
  var error = "<b>Some data is missing.</b><ul>";

  var mapCanvasTypeId = self.getMapCanvasTypeId();
  if (mapCanvasTypeId === "GOOGLE_MAPS_API") {
    if (!self.isLicenseAccepted()) {
      isValid = false;
      error += "<li>Please accept Google Maps License when choosing Google Maps API engine</li>";
    }
  }

  if (isValid) {
    return Promise.resolve(true);
  } else {
    return Promise.reject(new ValidationError(error));
  }
};

/**
 *
 */
EditProjectDialog.prototype.close = function () {
  var self = this;
  $(self.getElement()).dialog("close");
};

/**
 *
 * @param {number} overlayId
 * @returns {Promise}
 */
EditProjectDialog.prototype.saveOverlay = function (overlayId) {
  var self = this;
  var overlay = self._overlayById[overlayId];
  overlay.setName($("[name='name-" + overlayId + "']", self.getElement())[0].value);
  overlay.setDescription($("[name='description-" + overlayId + "']", self.getElement())[0].value);
  overlay.setPublicOverlay($("[name='publicOverlay-" + overlayId + "']", self.getElement())[0].checked);
  overlay.setDefaultOverlay($("[name='defaultOverlay-" + overlayId + "']", self.getElement())[0].checked);
  overlay.setCreator($("[name='creator-" + overlayId + "']", self.getElement())[0].value);

  return self.getServerConnector().updateOverlay(overlay);
};

/**
 *
 * @param {number} mapId
 * @returns {Promise}
 */
EditProjectDialog.prototype.saveMap = function (mapId) {
  var self = this;
  var map = self._mapsById[mapId];
  var centerX = parseInt($("[name='defaultCenterX-" + mapId + "']", self.getElement())[0].value);
  var centerY = parseInt($("[name='defaultCenterY-" + mapId + "']", self.getElement())[0].value);
  map.setDefaultCenterX(centerX);
  map.setDefaultCenterY(centerY);
  map.setDefaultZoomLevel($("[name='defaultZoomLevel-" + mapId + "']", self.getElement())[0].value);

  return self.getServerConnector().updateModel({projectId: self.getProject().getProjectId(), model: map});
};

/**
 *
 * @param {string} login
 * @returns {Promise}
 */
EditProjectDialog.prototype.saveUser = function (login) {
  var self = this;
  var checkboxes = $("[name='privilege-" + login + "']", self.getElement());
  var privileges = {};
  for (var i = 0; i < checkboxes.length; i++) {
    var checkbox = checkboxes[i];
    var privilege = {};
    privilege[self.getProject().getId()] = checkbox.checked;
    privileges[$(checkbox).attr("data")] = privilege;
  }

  return self.getServerConnector().updateUserPrivileges({
    user: self._userByLogin[login],
    privileges: privileges
  });
};

/**
 *
 * @param {number} overlayId
 * @returns {Promise}
 */
EditProjectDialog.prototype.removeOverlay = function (overlayId) {
  var self = this;
  return self.getServerConnector().removeOverlay({
    overlayId: overlayId
  }).then(function () {
    return self.refreshOverlays();
  });
};

/**
 *
 * @returns {Promise}
 */
EditProjectDialog.prototype.openAddOverlayDialog = function () {
  var self = this;
  if (self._addOverlayDialog !== undefined) {
    self._addOverlayDialog.destroy();
  }
  self._addOverlayDialog = new AddOverlayDialog({
    project: self.getProject(),
    customMap: null,
    element: document.createElement("div"),
    configuration: self.getConfiguration()
  });
  self._addOverlayDialog.addListener("onAddOverlay", function () {
    return self.refreshOverlays();
  });
  return self._addOverlayDialog.init().then(function () {
    return self._addOverlayDialog.open();
  });
};

/**
 *
 * @param {CommentsTab} commentsTab
 */
EditProjectDialog.prototype.setCommentsTab = function (commentsTab) {
  this._commentsTab = commentsTab;
};

/**
 *
 * @returns {CommentsTab}
 */
EditProjectDialog.prototype.getCommentsTab = function () {
  return this._commentsTab;
};

/**
 *
 * @returns {Promise}
 */
EditProjectDialog.prototype.refreshComments = function () {
  return this.getCommentsTab().refreshComments();
};


/**
 *
 */
EditProjectDialog.prototype.destroy = function () {
  var self = this;
  var div = self.getElement();
  var usersTable = $("[name=usersTable]", self.getElement())[0];
  var overlaysTable = $("[name=overlaysTable]", self.getElement())[0];
  var mapsTable = $("[name=mapsTable]", self.getElement())[0];
  if ($.fn.DataTable.isDataTable(usersTable)) {
    $(usersTable).DataTable().destroy();
  }
  if ($.fn.DataTable.isDataTable(overlaysTable)) {
    $(overlaysTable).DataTable().destroy();
  }
  if ($.fn.DataTable.isDataTable(mapsTable)) {
    $(mapsTable).DataTable().destroy();
  }

  if ($(div).hasClass("ui-dialog-content")) {
    $(div).dialog("destroy");
  }
  if (self._addOverlayDialog !== undefined) {
    self._addOverlayDialog.destroy();
    delete self._addOverlayDialog;
  }
  if (this.getCommentsTab() !== undefined) {
    this.getCommentsTab().destroy();
  }
};

/**
 *
 * @returns {string}
 */
EditProjectDialog.prototype.getMapCanvasTypeId = function () {
  return $("[name='project-map-canvas-type']", this.getElement()).val();
};

/**
 *
 * @param {boolean} isLicenseAccepted
 */
EditProjectDialog.prototype.setLicenseAccepted = function (isLicenseAccepted) {
  $("[name='project-google-maps-license']", this.getElement()).prop("checked", isLicenseAccepted);
};

/**
 *
 * @returns {boolean}
 */
EditProjectDialog.prototype.isLicenseAccepted = function () {
  return $("[name='project-google-maps-license']", this.getElement()).is(":checked");
};

module.exports = EditProjectDialog;
